/**********************************************************************************
 Persistence of Vision Ray Tracer Scene Description File
 File name   : HF_WaterAndFoam_test.pov
 Version     : 3.71 (UberPOV)
 Description : Water surface with a foam fringe along a height_field shore.
               Derived from code (2004) by Zeger Knaepen and Jaime Vives Piqueres.
               
 Date        : March 2017
 Author      : Thomas A. M. de Groot
 E-mail      : 
 
 Copyright (C) 2017. All rights reserved.
 Use and/or modification of this scene is free, except for commercial purposes.
 Commercial distribution is not allowed without written permission from the author.
**********************************************************************************/

// Render settings 3.7 (right-click on a line below):
// +w640 +h640 +a0.3 +am2 +bm2
// +w1200 +h750 +a0.3 +am2 +bm2 +wt6

//=================================================================================
// VERSION AND GAMMA SWITCHES
//=================================================================================
//#version 3.7;
#version unofficial patch 3.71; //UberPOV

//=================================================================================
// DEFAULTS AND INCLUDES
//=================================================================================
#default {pigment {rgb <1,0,0>}}
#default {finish {ambient 0.01}}

#include "colors.inc"
#include "rand.inc"
#include "functions.inc"
#include "transforms.inc"

//=================================================================================
// SCENE SWITCHES
//=================================================================================
#declare Rad            = on;		// Radiosity switch

#declare AspectRatio    = image_width/image_height;
#declare CamZoom        = 1;
#declare CamSky         = <0, 1, 0>;
#declare CamAng         = 70;	//39
#declare CamLoc         = <0.0, 100, -170.0>;
#declare CamLookAt      = <0.0, 0.0, 0.0>;


//=================================================================================
// GLOBAL SETTINGS
//=================================================================================
global_settings {
  assumed_gamma 1.0
  max_trace_level 5			          // default [5]
  #if (Rad)
  radiosity {
    pretrace_start 0.08           // start pretrace at this size
    pretrace_end   0.004          // end pretrace at this size
    count 35                      // higher -> higher quality (1..1600) [35]
    nearest_count 10, 5           // higher -> higher quality (1..20) [5]; (new in 3.7) optional 2nd parameter sets adaptive radiosity pretrace
    error_bound 1                 // higher -> smoother, less accurate [1.8]
    recursion_limit 1             // how much interreflections are calculated (1..5+) [3]
    low_error_factor .3           // reduce error_bound during last pretrace step
    gray_threshold 0.0            // increase for weakening colors (0..1) [0]
    minimum_reuse 0.015           // reuse of old radiosity samples [0.015]
    maximum_reuse 0.1							// new parameter 3.7 [0.2]
    brightness 1                  // brightness of radiosity effects (0..1) [1]

    adc_bailout 0.01/2
    normal off                    // take surface normals into account [off]
    media off                     // take media into account [off]
    always_sample off             // turn sampling in final trace off [on]
  }
  #end
}

//====================================================================================
// CAMERA AND ENVIRONMENT
//====================================================================================
camera {
  location  CamLoc
  sky       CamSky  
  up        CamSky
  direction z*CamZoom
  right     x*AspectRatio
  angle     CamAng
  look_at   CamLookAt
}

// sun ---------------------------------------------------------------------
light_source {<-1500, 2500, -2500> color White}

// sky ---------------------------------------------------------------------
sky_sphere { 
  pigment { 
    gradient y
    color_map {
      [0.00 rgb <1.0, 1.0, 1.0>]
      [0.30 rgb <0.0 ,0.1, 1.0>]
      [0.70 rgb <0.0, 0.1, 1.0>]
      [1.00 rgb <1.0, 1.0, 1.0>] 
    } 
    scale 2         
  }
}

// fog ---------------------------------------------------------------------
fog {
  fog_type   2
  distance   50
  color      White
  fog_offset 0.1
  fog_alt    2.0
  turbulence 0.8
}

// ground ------------------------------------------------------------------
plane { 
  y, -10 
  texture {
    pigment {color rgb <0.825,0.57,0.35>}
    normal {bumps 0.75 scale 0.025}
    //finish { phong 0.1 } 
  }
}

//====================================================================================
// HEIGHT_FIELD CODE
//====================================================================================
#declare HF_scale = <200, 50, 200>;
#declare HF_res   = 2000;

#declare F_HF_01 =
function {
  pigment {
    image_map {
      png "GC_MountainRidges.png" gamma 1.0
      map_type 0
      interpolate 2
    }
    warp {repeat x}
    warp {repeat y}
    scale 50
    warp {
      turbulence 0.2 
      octaves 1 //[6]
      lambda 1  //[2]
      omega 0.2 //[0.5]
    }
    scale 1/50
  }
}

#declare HF = 
height_field {
  function HF_res, HF_res {F_HF_01(x,y,z).hf} 
  smooth
  translate <-0.5, 0, -0.5>
  scale HF_scale
}

#declare Terrain =
intersection {
  object {HF}
  box {
    <-0.5, -0.5, -0.5>, <0.5, 0.5, 0.5>
    texture {pigment {Yellow}}
    scale HF_scale * <0.99, 10, 0.99>
  }
  texture {pigment {rgb Green*0.3}}
}

object {Terrain}

//====================================================================================
// WATER CODE
//====================================================================================
#declare Water             = on;
#declare Water_level       = 30;
#declare Absorption_amount = 5;
#declare Water_colour      = <0.8, 0.95, 1.0, 1.0>;
#declare Foam_turb         = 0.05;
#declare Foam_level        = 0.8;
#declare Foam_pattern      = object {HF translate Foam_level*y}


#declare T_water = 
texture {
	pigment {srgb Water_colour filter 1.0}
	normal {granite -0.0825 scale <1.5, 1.0, 1.0>*2}
	finish {
	  diffuse 0.1 
	  specular 0.5 
	  roughness 0.01
		reflection {0.01, 0.4 fresnel}
		conserve_energy
	}
}

#declare T_foam0 =
texture {
  pigment {srgb <0.95, 0.95, 0.80>*0.75}
	normal {bumps 1 scale 0.1}
	finish {
	  diffuse 0.7 
	  specular 0.3 
	  roughness 0.01
		reflection {0.01, 0.1 fresnel}
		conserve_energy
	}
}

#declare T_foam = 
texture {
  bozo
  texture_map {
	  [0.5 T_foam0]
	  [0.6 T_water]
  }
  scale 2
  warp {turbulence <1.0, 0.2, 0.1>}
}

#declare M_water =
material {
  texture {
    pigment_pattern {
    	object {Foam_pattern srgb 0, srgb 1}
    }
    texture_map {
      [0.0 T_water]
      [1.0 T_foam]
    }
    scale 1/10 warp {turbulence Foam_turb lambda 3} scale 10
  }
}

#if (Water)
box {
	-0.55, <0.55, 0.0, 0.55>
	scale HF_scale
	translate Water_level*y
	hollow
	material {M_water 
		interior {
			ior 4/3
			//fade_distance 0.001 fade_power 20
			media {
				absorption Absorption_amount*(1-Water_colour)
			}
		}
	}
}
#end

//====================================================================================










